<?php
/**
 * Woo Invoice Pro Plugin Uses Tracker
 * Uses Webappick Insights for tracking
 *
 * @since   1.2.2
 * @version 1.0.0
 */
if ( ! defined( 'ABSPATH' ) ) {
	die();
}

if ( ! class_exists(
	'

'
)
) {
	/**
	 * Class Woo_Invoice_ProWebAppickAPI
	 */
	final class Woo_Invoice_ProWebAppickAPI {


		/**
		 * Singleton instance
		 *
		 * @var Woo_Invoice_ProWebAppickAPI
		 */
		protected static $instance;

		/**
		 * @var WebAppick\AppServices\Client
		 */
		protected $client = null;

		/**
		 * @var WebAppick\AppServices\Insights
		 */
		protected $insights = null;

		/**
		 * Promotions Class Instance
		 *
		 * @var WebAppick\AppServices\Promotions
		 */
		public $promotion = null;

		/**
		 * Plugin License Manager
		 *
		 * @var WebAppick\AppServices\License
		 */
		protected $license = null;

		/**
		 * Plugin Updater
		 *
		 * @var WebAppick\AppServices\Updater
		 */
		protected $updater = null;

		/**
		 * Initialize
		 *
		 * @return Woo_Invoice_ProWebAppickAPI
		 */
		public static function getInstance() {
			if ( is_null( self::$instance ) ) {
				self::$instance = new self();
			}
			return self::$instance;
		}

		/**
		 * Class constructor
		 *
		 * @return void
		 * @since  1.0.0
		 */
		private function __construct() {
			if ( ! class_exists( 'WebAppick\AppServices\Client' ) ) {
				/**
		  * @noinspection PhpIncludeInspection
*/
				include_once CHALLAN_PRO_LIBS_PATH . 'WebAppick/AppServices/Client.php';
			}
			// Load Client
			$this->client = new WebAppick\AppServices\Client( '732d63fc-a21a-4826-80eb-7959df702cda', 'Challan Pro', CHALLAN_PRO_ROOT_FILE );
			// Load
			$this->insights  = $this->client->insights(); // Plugin Insights
			$this->license   = $this->client->license(); // Plugin License Checker
			$this->updater   = $this->client->updater(); // Plugin Update Checker
			$this->promotion = $this->client->promotions(); // Promo offers

			$this->license->set_product_data(
				array(
					'yearly'   => array(
						'label'    => __( 'Yearly', 'webappick-pdf-invoice-for-woocommerce' ),
						'products' => array(
							51374 => __( 'Single Site Yearly', 'webappick-pdf-invoice-for-woocommerce' ),
							51373 => __( 'Five Sites Yearly', 'webappick-pdf-invoice-for-woocommerce' ),
							51375 => __( 'Ten Sites Yearly', 'webappick-pdf-invoice-for-woocommerce' ),
						),
					),
					'lifetime' => array(
						'label'    => __( 'Lifetime', 'webappick-pdf-invoice-for-woocommerce' ),
						'products' => array(
							63682 => __( 'Single Site Lifetime', 'webappick-pdf-invoice-for-woocommerce' ),
							63683 => __( 'Five Sites Lifetime', 'webappick-pdf-invoice-for-woocommerce' ),
							63684 => __( 'Ten Sites Lifetime', 'webappick-pdf-invoice-for-woocommerce' ),
						),
					),
				)
			);

			// Setup
			$this->license->add_settings_page(
				array(
					'menu_title'  => __( 'License', 'webappick-pdf-invoice-for-woocommerce' ),
					'type'        => 'submenu',
					'parent_slug' => 'webappick-woo-invoice-settings',
				)
			);
			$this->promotion->set_source( 'https://api.bitbucket.org/2.0/snippets/woofeed/RLbyop/files/woo-feed-notice.json' );
			// Initialize
			$this->insightInit();
			$this->license->init();
			$this->updater->init();
			$this->promotion->init();

			// Housekeeping
			// migrate to new license from API Manager
			add_action( 'init', array( $this, '__migrate_wc_am_license' ), 999 );
			// Filter updater api data
			add_filter( 'WebAppick_' . $this->client->getSlug() . '_plugin_api_info', array( $this, '__plugin_api_info' ), 10, 1 );
		}

		/**
		 * Cloning is forbidden.
		 *
		 * @since 1.0.2
		 */
		public function __clone() {
			_doing_it_wrong( __FUNCTION__, esc_html__( 'Cloning is forbidden.', 'webappick-pdf-invoice-for-woocommerce' ), '1.0.2' );
		}

		/**
		 * Check license status
		 *
		 * @return bool
		 */
		public function isActive() {
			return $this->license->is_valid();
		}

		/**
		 * Initialize Insights
		 *
		 * @return void
		 */
		private function insightInit() {
			$this->insights->add_extra(
				array(
					'products'   => $this->insights->get_post_count( 'product' ),
					'variations' => $this->insights->get_post_count( 'product_variation' ),
					'orders'     => $this->insights->get_post_count( 'order' ),
				)
			);
			$projectSlug = $this->client->getSlug();
			add_filter( $projectSlug . '_what_tracked', array( $this, 'data_we_collect' ), 10, 1 );
			add_filter(
				"WebAppick_{$projectSlug}_Support_Ticket_Recipient_Email",
				function () {
					return 'support@webappick.com';
				},
				10
			);
			add_filter( "WebAppick_{$projectSlug}_Support_Ticket_Email_Template", array( $this, 'supportTicketTemplate' ), 10 );
			add_filter( "WebAppick_{$projectSlug}_Support_Request_Ajax_Success_Response", array( $this, 'supportResponse' ), 10 );
			add_filter( "WebAppick_{$projectSlug}_Support_Request_Ajax_Error_Response", array( $this, 'supportErrorResponse' ), 10 );
			add_filter(
				"WebAppick_{$projectSlug}_Support_Page_URL",
				function () {
					return 'https://webappick.com/my-account/contact-support/';
				},
				10
			);
			$this->insights->init();
		}

		/**
		 * Generate Support Ticket Email Template
		 *
		 * @return string
		 */
		public function supportTicketTemplate() {
			// dynamic variable format __INPUT_NAME__
			$licenseData = '';
			if ( $this->license->is_valid() ) {
				$licenseData = sprintf( '<br>API Key : %s', $this->license->get_key() );
			}
			/**
	   * @noinspection HtmlUnknownTarget
*/
			$template  = '<div style="margin: 10px auto;"><p>Website : <a href="__WEBSITE__">__WEBSITE__</a><br>Plugin : %s (v.%s)%s</p></div>';
			$template  = sprintf( $template, $this->client->getName(), $this->client->getProjectVersion(), $licenseData );
			$template .= '<div style="margin: 10px auto;"><hr></div>';
			$template .= '<div style="margin: 10px auto;"><h3>__SUBJECT__</h3></div>';
			$template .= '<div style="margin: 10px auto;">__MESSAGE__</div>';
			$template .= '<div style="margin: 10px auto;"><hr></div>';
			$template .= sprintf(
				'<div style="margin: 50px auto 10px auto;"><p style="font-size: 12px;color: #009688">%s</p></div>',
				'Message Processed With WebAppick Service Library (v.' . $this->client->getClientVersion() . ')'
			);
			return $template;
		}

		/**
		 * Generate Support Ticket Ajax Response
		 *
		 * @return string
		 */
		public function supportResponse() {
			$response        = '';
			$response       .= sprintf( '<h3>%s</h3>', esc_html__( 'Thank you -- Support Ticket Submitted.', 'webappick-pdf-invoice-for-woocommerce' ) );
			$ticketSubmitted = esc_html__( 'Your ticket has been successfully submitted.', 'webappick-pdf-invoice-for-woocommerce' );
			$twenty4Hours    = sprintf( '<strong>%s</strong>', esc_html__( '24 hours', 'webappick-pdf-invoice-for-woocommerce' ) );
			$notification    = sprintf( esc_html__( 'You will receive an email notification from "support@webappick.com" in your inbox within %s.', 'webappick-pdf-invoice-for-woocommerce' ), $twenty4Hours ); // phpcs:ignore
			$followUp        = esc_html__( 'Please Follow the email and WebAppick Support Team will get back with you shortly.', 'webappick-pdf-invoice-for-woocommerce' );
			$response       .= sprintf( '<p>%s %s %s</p>', $ticketSubmitted, $notification, $followUp );
			$docLink         = sprintf( '<a class="button button-primary" href="https://webappick.helpscoutdocs.com/" target="_blank"><span class="dashicons dashicons-media-document" aria-hidden="true"></span> %s</a>', esc_html__( 'Documentation', 'webappick-pdf-invoice-for-woocommerce' ) );
			$vidLink         = sprintf( '<a class="button button-primary" href="http://bit.ly/2u6giNz" target="_blank"><span class="dashicons dashicons-video-alt3" aria-hidden="true"></span> %s</a>', esc_html__( 'Video Tutorials', 'webappick-pdf-invoice-for-woocommerce' ) );
			$response       .= sprintf( '<p>%s %s</p>', $docLink, $vidLink );
			$response       .= '<br><br><br>';
			$toc             = sprintf( '<a href="https://webappick.com/terms-and-conditions/" target="_blank">%s</a>', esc_html__( 'Terms & Conditions', 'webappick-pdf-invoice-for-woocommerce' ) );
			$pp              = sprintf( '<a href="https://webappick.com/privacy-policy/" target="_blank">%s</a>', esc_html__( 'Privacy Policy', 'webappick-pdf-invoice-for-woocommerce' ) );
			$policy          = sprintf( esc_html__( 'Please read our %1$s and %2$s', 'webappick-pdf-invoice-for-woocommerce' ), $toc, $pp ); //phpcs:ignore
			$response       .= sprintf( '<p style="font-size: 12px;">%s</p>', $policy );
			return $response;
		}

		/**
		 * Set Error Response Message For Support Ticket Request
		 *
		 * @return string
		 */
		public function supportErrorResponse() {
			return sprintf(
				'<div class="mui-error"><p>%s</p><p>%s</p><br><br><p style="font-size: 12px;">%s</p></div>',
				esc_html__( 'Something Went Wrong. Please Try The Support Ticket Form On Our Website.', 'webappick-pdf-invoice-for-woocommerce' ),
				sprintf( '<a class="button button-primary" href="https://webappick.com/support/" target="_blank">%s</a>', esc_html__( 'Get Support', 'webappick-pdf-invoice-for-woocommerce' ) ),
				esc_html__( 'Support Ticket form will open in new tab in 5 seconds.', 'webappick-pdf-invoice-for-woocommerce' )
			);
		}

		/**
		 * Set Data Collection description for the tracker
		 *
		 * @param $data
		 *
		 * @return array
		 */
		public function data_we_collect( $data ) {
			$data = array_merge(
				$data,
				array(
					esc_html__( 'Number of products in your site.', 'webappick-pdf-invoice-for-woocommerce' ),
					esc_html__( 'Number of Orders in your site.', 'webappick-pdf-invoice-for-woocommerce' ),
					esc_html__( 'Site name, language and url.', 'webappick-pdf-invoice-for-woocommerce' ),
					esc_html__( 'Number of active and inactive plugins.', 'webappick-pdf-invoice-for-woocommerce' ),
					esc_html__( 'Your name and email address.', 'webappick-pdf-invoice-for-woocommerce' ),
				)
			);

			return $data;
		}

		public function get_data_collection_description() {
			return $this->insights->get_data_collection_description();
		}

		/**
		 * Add Missing Info for plugin details after fetching through api
		 *
		 * @param  $data
		 * @return array
		 */
		public function __plugin_api_info( $data ) {
			// house keeping
			if ( isset( $data['homepage'], $data['author'] ) && false === strpos( $data['author'], '<a' ) ) {
				/**
		  * @noinspection HtmlUnknownTarget
*/
				$data['author'] = sprintf( '<a href="%s">%s</a>', $data['homepage'], $data['author'] );
			}
			if ( ! isset( $data['contributors'] ) ) {
				$data['contributors'] = array(
					'wahid0003' => array(
						'profile'      => 'https://webappick.com/',
						'avatar'       => 'https://secure.gravatar.com/avatar/3fd731922ac65c5a1ba045b28a7bfb21?s=96&d=monsterid&r=g',
						'display_name' => 'Md Ohidul Islam',
					),
				);
			}
			$sections = array( 'description', 'installation', 'faq', 'screenshots', 'changelog', 'reviews', 'other_notes' );
			foreach ( $sections as $section ) {
				if ( isset( $data['sections'][ $section ] ) && empty( $data['sections'][ $section ] ) ) {
					unset( $data['sections'][ $section ] );
				}
			}

			return $data;
		}

		/**
		 * Update Tracker OptIn
		 *
		 * @param  bool $override optional. ignore last send datetime settings if true.
		 * @return void
		 * @see    Insights::send_tracking_data()
		 */
		public function trackerOptIn( $override = false ) {
			$this->insights->optIn( $override );
		}

		/**
		 * Update Tracker OptOut
		 *
		 * @return void
		 */
		public function trackerOptOut() {
			$this->insights->optOut();
		}

		/**
		 * Check if tracking is enable
		 *
		 * @return bool
		 */
		public function is_tracking_allowed() {
			return $this->insights->is_tracking_allowed();
		}

		/**
		 * Migrate to New License Client Format from WooCommerce API Manager Data
		 *
		 * @param  bool $override
		 * @return bool
		 */
		public function __migrate_wc_am_license( $override = false ) {
			// if license isn't valid try to migrate license data from wc-am.
			if ( $this->license->is_valid() && ! $override ) {
				return false;
			}
			return $this->license->migrate_license_from_wc_am( $override );
		}
	}
}
// End of file class-woo-invoice-webappick-api.php
